import * as ts from 'typescript';
import * as path from 'path';
import { readFileSync } from 'fs';

export type Info = {
    componentName: string;
    makeParameters: ts.NodeArray<ts.ParameterDeclaration>;
    actFuncs: ts.MethodSignature[];
    hasAct: boolean;
    hasState: boolean;
}

export function extractInfo(fileName: string): Info {
    let sourceFile = ts.createSourceFile(
        fileName,
        readFileSync(fileName).toString(),
        ts.ScriptTarget.ES2015,
        true
    );

    let makeParameters: ts.NodeArray<ts.ParameterDeclaration>;
    let actFuncs: ts.MethodSignature[];
    let hasAct = false;
    let hasState = false;

    function visit(sourceFile: ts.SourceFile) {
        traverse(sourceFile);

        function traverse(node: ts.Node) {
            switch (node.kind) {
                case ts.SyntaxKind.VariableDeclaration:
                    const varDeclr = node as ts.VariableDeclaration;
                    if (varDeclr.name.getText() == 'make' && varDeclr.initializer.kind == ts.SyntaxKind.ArrowFunction) {
                        const fnType = varDeclr.initializer as ts.ArrowFunction;
                        makeParameters = fnType.parameters;
                    }
                    return;

                case ts.SyntaxKind.TypeAliasDeclaration:
                    const aliasDeclr = node as ts.TypeAliasDeclaration;
                    const typ = aliasDeclr.type;
                    if (aliasDeclr.name.getText() == 'Action' && typ.kind == ts.SyntaxKind.TypeLiteral) {
                        hasAct = true;
                        const litTyp = typ as ts.TypeLiteralNode;
                        actFuncs = litTyp.members.filter(m => m.kind == ts.SyntaxKind.MethodSignature) as ts.MethodSignature[];
                    } else if (aliasDeclr.name.getText() == 'State') {
                        hasState = true;
                    }
                    return;
            }

            ts.forEachChild(node, traverse);
        }
    }

    visit(sourceFile);

    const componentName = path.parse(fileName).name;

    return {
        componentName,
        makeParameters,
        actFuncs,
        hasAct,
        hasState
    };
}
